//
// C++ Implementation:
//
// Description:
//
//
// Author: Jean-Michel PETIT <k9copy@free.fr>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//


#include "k9import.h"
#include "k9title.h"
#include "k9avifile.h"
#include "k9lvitemimport.h"
#include <klocale.h>
#include "k9menuedit.h"
#include "k9config.h"
#include "k9burndvd.h"
#include <qdir.h>
#include <kfiledialog.h>
#include <kcombobox.h>
#include <kiconloader.h>
#include "k9processlist.h"
#include "k9importfiles.h"
#include <QDockWidget>
k9Import::k9Import(QWidget* parent, const char* name,k9CdDrives *_drives)
        : QWidget(parent) {
    Ui_import.setupUi(this);
    setWindowTitle(KDialog::makeStandardCaption(i18n("import"),this));
    m_parent=(k9ImportFiles*) parent;
    drives=_drives;

    connect(&m_newDVD,SIGNAL(sigAddTitle()),this,SLOT(titleAdded()));
    lvDVD=Ui_import.lvDVD;
    connect(lvDVD,SIGNAL(currentItemChanged ( QTreeWidgetItem * , QTreeWidgetItem *)),this,SLOT(lvDVDCurrentChanged( QTreeWidgetItem *,QTreeWidgetItem *) ));


}

k9Import::~k9Import() {}



/*$SPECIALIZATION$*/

void k9Import::init() {
    m_root=new QTreeWidgetItem(lvDVD);
    m_root->setText(0,i18n("Root Menu"));
    m_root->setExpanded(true);
    m_root->setIcon(0,SmallIcon("dvd_unmount"));
    emit aviFileSelected(NULL);

    connect(drives,SIGNAL(deviceAdded( k9CdDrive*)),this,SLOT(deviceAdded( k9CdDrive* )));
    connect(drives,SIGNAL(deviceRemoved( k9CdDrive*)),this,SLOT(deviceRemoved( k9CdDrive* )));

    readDrives();
    m_root->setSelected(true);
    emit rootSelected(&m_newDVD);
    emit aviFileSelected(NULL);

}

void k9Import::readDrives() {
    Ui_import.cbOutputDev->clear();
    recorderList.clear();
    Ui_import.cbOutputDev->addItem(i18n("ISO Image"));
    Ui_import.cbOutputDev->addItem(i18n("Folder"));

    drives->scanDrives();

}

void k9Import::deviceAdded(k9CdDrive *_drive) {
    addDrive(_drive);
}

void k9Import::deviceRemoved(k9CdDrive *_drive) {

    int i=recorderList.indexOf(_drive);
    recorderList.removeAll( _drive);
    Ui_import.cbOutputDev->removeItem(i+2);

}

void k9Import::addDrive(k9CdDrive *drive) {
#ifdef HAVE_HAL
    if (drive->getDevice()!=NULL) {
        k9HalDevice *dev=drive->getDevice();
        connect(dev,SIGNAL(volumeChanged( const QString &,const QString &)),this,SLOT(volumeChanged(const QString&,const QString&)));
    }
#endif
    if (drive->canWriteDVD) {
        Ui_import.cbOutputDev->addItem(drive->name+ "  ("+drive->device+")",-1);
        recorderList.append(drive);
    }

}

void k9Import::volumeChanged(const QString &device,const QString &volumeName) {}



QTreeWidgetItem * k9Import::getRoot() const {
    return m_root;
}


void k9Import::aviFileUpdated(k9AviFile *_aviFile) {
    for (int i=0;i< m_root->childCount();i++) {
	k9LvItemImport *itemTitle = (k9LvItemImport*)m_root->child(i);
        for (int j=0;j<itemTitle->childCount();j++) {
            k9LvItemImport *itemChapter = (k9LvItemImport*)itemTitle->child(j);
            if (itemChapter->getAviFile()==_aviFile) {
                itemChapter->setText(1,_aviFile->getStart().toString("hh:mm:ss") +" - "+_aviFile->getEnd().toString("hh:mm:ss"));
                itemChapter->setIcon(0,QIcon(QPixmap::fromImage(_aviFile->getImage().scaled(50,40))));
            }
        }
    }
    updateTotalTime();
}

void k9Import::buttonUpdated(k9MenuButton *_button, const QImage &_image) {
    for (int i=0;i< m_root->childCount();i++) {
	k9LvItemImport *itemTitle = (k9LvItemImport*)m_root->child(i);
        if (itemTitle->getTitle()->getButton()==_button) {
            QImage img=_image.scaled(50,40,Qt::IgnoreAspectRatio);
            QPixmap pix=QPixmap::fromImage(img);
            itemTitle->setIcon(0,QIcon(pix));
        }
    }
}



void k9Import::lvDVDCurrentChanged( QTreeWidgetItem *_item,QTreeWidgetItem *)  {
    k9LvItemImport *item=(k9LvItemImport*)_item;

    switch (_item->type()) {
    case 1002:
        emit aviFileSelected(item->getAviFile());
        emit titleSelected(item->getTitle());
        break;
    case 1001:
        emit titleSelected(item->getTitle());
	emit aviFileSelected(NULL);
        break;
    default:
        emit rootSelected(&m_newDVD);
	emit aviFileSelected(NULL);
        break;
    }
}

void k9Import::setProgressWindow(QWidget *_widget) {
/*    m_toolView=m_parent->setToolWindow(_widget,KDockWidget::DockRight,i18n("processing"),i18n("processing"));
    m_dockWidget = m_parent->getVisibleDock();
    m_parent->setActions( false);
    m_toolView->show();
//    this->setEnabled(false);
*/
}

void k9Import::removeProgressWindow() {
/*    m_parent->setActions(true);

    m_parent->removeToolWindow( m_toolView);
    this->setEnabled(true);
    if (m_dockWidget!=NULL)
        m_dockWidget->changeHideShowState();
*/
}

void k9Import::execute() {
    QString filename;
    k9Config config;
    QString outputDir=config.getPrefOutput();
    if (Ui_import.cbOutputDev->currentIndex() ==0) {
        filename=KFileDialog::getSaveFileName (QDir::homePath(),"*.iso", 0,i18n("Save image to disk"));
        if (filename =="")
            return;
    } else if (Ui_import.cbOutputDev->currentIndex()==1) {
       outputDir= KFileDialog::getExistingDirectory(QDir::homePath());
       if (outputDir=="")
       return;
    }
    k9ProcessList *p=new k9ProcessList(QApplication::activeWindow());

    p->setModal(true);
    p->show();
    setProgressWindow(p);
    m_newDVD.setProcessList(p);
    m_newDVD.setWorkDir(outputDir);
    m_newDVD.execute();
    p->close();
    bool burn=(m_newDVD.getError()=="") ;
    if (burn && Ui_import.cbOutputDev->currentIndex()!=1) {
        //changeStatusbar(i18n("Burning DVD"),sbMessage);

        k9BurnDVD b;

        setProgressWindow( b.getDialog());
        b.setworkDir(config.getPrefOutput() );
        b.setUseK3b(config.getPrefK3b() );
        b.setAutoBurn(config.getPrefAutoBurn());
        b.setvolId("DVD");
        b.setSpeed( i18n("default"));
        if (Ui_import.cbOutputDev->currentIndex() !=0) {
            k9CdDrive * drive=(k9CdDrive*)recorderList.at(Ui_import.cbOutputDev->currentIndex()-2);
            b.setburnDevice(drive->device);
        } else
            b.makeIso(filename);
        b.burn();
        removeProgressWindow();
    }
  //  m_parent->removeDockWidget(doc);
}


void k9Import::cbFormatActivated(const QString &_format) {
    if (_format == "PAL") {
        m_newDVD.setFormat(k9NewDVD::PAL);
        m_menuEdit->setFormat(k9MenuEdit::PAL);
    } else {
        m_newDVD.setFormat(k9NewDVD::NTSC);
        m_menuEdit->setFormat(k9MenuEdit::NTSC);
    }

}

void k9Import::setMenuEdit(k9MenuEdit* _value) {
    m_menuEdit = _value;
}

void k9Import::setEnableCreate(bool _state) {
//    m_parent->setEnabledCreateDVD(true);
}

void k9Import::titleAdded() {
    emit rootSelected(&m_newDVD);
}

void k9Import::updateTotalTime() {
    int total=m_newDVD.getTotalTime();
    Ui_import.gsTotal->setValue(total/60);
    QTime t(0,0,0);
    t=t.addSecs(total);
    Ui_import.lTotal->setText(t.toString("hh:mm:ss"));
}
